@@ -39,15 +39,16 @@ except:
 import os
 import getopt
 from decimal import Decimal, getcontext, ROUND_HALF_EVEN
-from numpy import *
-from numpy.linalg import *
+import numpy as np
+import numpy.ma as ma
+from weightedstats import weighted_median
 
 __title__      = "pyconsensus"
-__version__    = "0.1.6"
+__version__    = "0.2"
 __author__     = "Paul Sztorc and Jack Peterson"
 __license__    = "GPL"
 __maintainer__ = "Jack Peterson"
-__email__      = "jack@dyffy.com"
+__email__      = "jack@tinybike.net"
 
 # Python 3 compatibility
 from six.moves import xrange as range
@@ -66,35 +67,13 @@ class Oracle(object):
 
     def __init__(self, votes=None, decision_bounds=None, weights=-1,
                  catch_p=.1, max_row=5000, verbose=False):
-        self.votes = ma.masked_array(votes, isnan(votes))
+        self.votes = ma.masked_array(votes, np.isnan(votes))
         self.decision_bounds = decision_bounds
         self.weights = weights
         self.catch_p = catch_p
         self.max_row = max_row
         self.verbose = verbose
 
-    def WeightedMedian(self, data, weights):
-        """Calculate a weighted median.
-
-        Args:
-          data (list or numpy.array): data
-          weights (list or numpy.array): weights
-
-        """
-        data, weights = array(data).squeeze(), array(weights).squeeze()
-        s_data, s_weights = map(array, zip(*sorted(zip(data, weights))))
-        midpoint = 0.5 * sum(s_weights)
-        if any(weights > midpoint):
-            w_median = median(data[weights == max(weights)])
-        else:
-            cs_weights = cumsum(s_weights)
-            idx = where(cs_weights <= midpoint)[0][-1]
-            if cs_weights[idx] == midpoint:
-                w_median = mean(s_data[idx:idx+2])
-            else:
-                w_median = s_data[idx+1]
-        return w_median
-
     def Rescale(self, UnscaledMatrix, Scales):
         """Forces a matrix of raw (user-supplied) information
         (for example, # of House Seats, or DJIA) to conform to
@@ -116,29 +95,34 @@ class Oracle(object):
             OutMatrix[:,i] -= Scales[i]["min"]
 
         # Rescale
-        NaIndex = isnan(OutMatrix)
+        NaIndex = np.isnan(OutMatrix)
         OutMatrix[NaIndex] = 0
 
-        OutMatrix = dot(OutMatrix, diag(InvSpan))
-        OutMatrix[NaIndex] = nan
+        OutMatrix = np.dot(OutMatrix, np.diag(InvSpan))
+        OutMatrix[NaIndex] = np.nan
 
         return OutMatrix
 
     def MeanNa(self, Vec):
         """Takes masked array, replaces missing values with array mean."""
-        MM = mean(Vec)
-        Vec[where(Vec.mask)] = MM
-        return(Vec)
+        MM = np.mean(Vec)
+        Vec[np.where(Vec.mask)] = MM
+        return Vec
         
     def GetWeight(self, Vec, AddMean=0):
-        """Takes an array (vector in practice), and returns proportional distance from zero."""
-        New = abs(Vec)       #Absolute Value
-        if AddMean == 1:     #Add the mean to each element of the vector
-            New = New + mean(New)
-        if sum(New) == 0:    #Catch an error here
-            New = New + 1
-        New = New/sum(New)   #Normalize
-        return(New)
+        """Takes an array, and returns proportional distance from zero."""
+        New = abs(Vec)
+
+        # Add the mean to each element of the vector
+        if AddMean == 1:
+            New += np.mean(New)
+
+        # Catch an error here
+        if np.sum(New) == 0:
+            New += 1
+
+        # Normalize
+        return New / np.sum(New)
 
     def Catch(self, X, Tolerance=0):
         """Forces continuous values into bins at 0, .5, and 1"""
@@ -150,119 +134,128 @@ class Oracle(object):
             return .5
 
     def Influence(self, Weight):
-        """Takes a normalized Vector (one that sums to 1), and computes relative strength of the indicators."""
+        """Takes a normalized Vector (one that sums to 1), and computes
+        relative strength of the indicators.
+        """
         N = len(Weight)
-        Expected = [[1/N]]*N
+        Expected = [[1/N]] * N
         Out = []
         for i in range(1, N):
-            Out.append(Weight[i]/Expected[i])
-        return(Out)
+            Out.append(Weight[i] / Expected[i])
+        return Out
 
     def ReWeight(self, Vec):
         """Get the relative influence of numbers, treat NaN as influence-less."""
         Out = Vec
-        Exclude = isnan(Vec)
-        Out[Exclude] = 0             # set missing to 0
-        Out = Out / sum(Out)  # normalize
-        return Out
+        Exclude = np.isnan(Vec)
 
-    def DemocracyCoin(self, Mat):
-        """For testing, easier to assume uniform coin distribution."""
-        # print("NOTE: No coin distribution given, assuming democracy [one row, one vote].")
-        Rep = self.GetWeight( array([[1]]*len(Mat) )) #Uniform weights if none were provided.
-        return Rep
+        # Set missing values to 0
+        Out[Exclude] = 0
 
-    def WeightedCov(self, Mat, Rep=-1):
-        """Takes 1] a masked array, and 2] an [n x 1] dimentional array of weights, and computes the weighted covariance
-        matrix and center of a given array.
-        Taken from http://stats.stackexchange.com/questions/61225/correct-equation-for-weighted-unbiased-sample-covariance"""
-        if type(Rep) is int:
-            Rep = self.DemocracyCoin(Mat)
+        # Normalize
+        return Out / np.sum(Out)
 
-        Coins = ma.copy(Rep)
-        for i in range(len(Rep)):
-            Coins[i] = (int( (Rep[i] * 1000000)[0] )) 
+    def DemocracyCoin(self, votes):
+        """No coin distribution given, assuming democracy: one person, one vote."""
+        return self.GetWeight(np.array([[1]] * len(votes)))
+
+    def WeightedCov(self, votes, reputation=-1):
+        """Takes 1] a masked array, and 2] an [n x 1] dimentional array of
+        weights, and computes the weighted covariance matrix and center of
+        a given array.
+        Taken from
+        http://stats.stackexchange.com/questions/61225/correct-equation-for-weighted-unbiased-sample-covariance
+        """
+        if type(reputation) is int:
+            reputation = self.DemocracyCoin(votes)
+
+        Coins = np.array([map(int, i) for i in reputation * 10**6])
 
         # Computing the weighted sample mean (fast, efficient and precise)
-        Mean = ma.average(Mat, axis=0, weights=hstack(Coins))
-        XM = matrix( Mat-Mean ) # xm = X diff to mean
+        weighted_sample_mean = ma.average(votes, axis=0, weights=np.hstack(Coins))
+        XM = np.matrix(votes - weighted_sample_mean) # xm = X diff to mean
 
         # Compute the unbiased weighted sample covariance
-        sigma2 = matrix( 1/(sum(Coins)-1) * ma.multiply(XM, Coins).T.dot(XM) );
+        sigma2 = np.matrix(1/(np.sum(Coins)-1) * ma.multiply(XM, Coins).T.dot(XM));
 
-        return {'Cov': array(sigma2), 'Center': array(XM)}
+        return {'Cov': np.array(sigma2), 'Center': np.array(XM)}
 
-    def WeightedPrinComp(self, Mat, Rep=-1):
+    def WeightedPrinComp(self, votes, reputation=-1):
         """Takes a matrix and row-weights and manually computes the statistical procedure known as Principal Components Analysis (PCA)
         This version of the procedure is so basic, that it can also be thought of as merely a singular-value decomposition on a weighted covariance matrix."""      
-        wCVM = self.WeightedCov(Mat,Rep)
-        SVD = svd(wCVM['Cov'])
+        wCVM = self.WeightedCov(votes, reputation)
+        SVD = np.linalg.svd(wCVM['Cov'])
 
-        L = SVD[0].T[0]                      #First loading
-        S = dot(wCVM['Center'],SVD[0]).T[0]   #First Score
+        # First loading
+        L = SVD[0].T[0]
 
-        return (L,S)
+        # First score
+        S = np.dot(wCVM['Center'], SVD[0]).T[0]
 
-    def GetRewardWeights(self, M, Rep=-1, Alpha=.1, Verbose=False):
-        """
-        Calculates new reputations using a weighted
+        return L, S
+
+    def GetRewardWeights(self, votes, reputation=-1, Alpha=.1, Verbose=False):
+        """Calculates new reputations using a weighted
         Principal Component Analysis (PCA).
 
         """
-        if type(Rep) is int:
-            Rep = self.DemocracyCoin(M)
-                
-        Results = self.WeightedPrinComp(M, Rep)        
-        Rep = self.GetWeight(Rep)  #Need the old reputation back for the rest of this.
-        FirstLoading = Results[0] #The first loading is designed to indicate which Decisions were more 'agreed-upon' than others. 
-        FirstScore   = Results[1] #The scores show loadings on consensus (to what extent does this observation represent consensus?)
+        if type(reputation) is int:
+            reputation = self.DemocracyCoin(votes)
+
+        Results = self.WeightedPrinComp(votes, reputation)
+        # Need the old reputation back for the rest of this.
+        reputation = self.GetWeight(reputation)
+        # The first loading is designed to indicate which Decisions were more 'agreed-upon' than others.
+        FirstLoading = Results[0]
+        # The scores show loadings on consensus (to what extent does this observation represent consensus?)
+        FirstScore = Results[1]
 
         #PCA, being an abstract factorization, is incapable of determining anything absolute.
         #Therefore the results of the entire procedure would theoretically be reversed if the average state of Decisions changed from TRUE to FALSE.
         #Because the average state of Decisions is a function both of randomness and the way the Decisions are worded, I quickly check to see which
         #  of the two possible 'new' reputation vectors had more opinion in common with the original 'old' reputation.
         #  I originally tried doing this using math but after multiple failures I chose this ad hoc way.
-        Set1 =  FirstScore+abs(min(FirstScore))
-        Set2 =  FirstScore-max(FirstScore)         
-        Old = dot(Rep.T,M)
-        New1 = dot(self.GetWeight(Set1), M)
-        New2 = dot(self.GetWeight(Set2), M)
+        Set1 = FirstScore + abs(min(FirstScore))
+        Set2 = FirstScore - max(FirstScore)
+        Old = np.dot(reputation.T, votes)
+        New1 = np.dot(self.GetWeight(Set1), votes)
+        New2 = np.dot(self.GetWeight(Set2), votes)
 
-        #Difference in Sum of squared errors, if >0, then New1 had higher errors (use New2), and conversely if <0 use 1.
-        RefInd = sum( (New1-Old)**2) -  sum( (New2-Old)**2)
-        if(RefInd <= 0):
+        # Difference in sum of squared errors. If > 0, then New1 had higher
+        # errors (use New2); conversely if < 0, then use New1.
+        RefInd = np.sum((New1 - Old)**2) - np.sum((New2 - Old)**2)
+        if RefInd <= 0:
             AdjPrinComp = Set1
-        if(RefInd > 0):
+        if RefInd > 0:
             AdjPrinComp = Set2
-        
+
         if Verbose:
             print("")
-            print("Estimations using: Previous Rep, Option 1, Option 2")
-            print( vstack([Old, New1, New2]).T )
+            print("Estimations using: Previous reputation, Option 1, Option 2")
+            print(np.vstack([Old, New1, New2]).T)
             print("")
             print("Previous period reputations, Option 1, Option 2, Selection")
-            print( vstack([ Rep.T, Set1, Set2, AdjPrinComp]).T )
+            print(np.vstack([reputation.T, Set1, Set2, AdjPrinComp]).T)
       
         #Declared here, filled below (unless there was a perfect consensus).
-        RowRewardWeighted = Rep # (set this to uniform if you want a passive diffusion toward equality when people cooperate [not sure why you would]). Instead diffuses towards previous reputation (Smoothing does this anyway).
-        if(max(abs(AdjPrinComp))!=0):
+        RowRewardWeighted = reputation # (set this to uniform if you want a passive diffusion toward equality when people cooperate [not sure why you would]). Instead diffuses towards previous reputation (Smoothing does this anyway).
+        if max(abs(AdjPrinComp)) != 0:
             # Overwrite the inital declaration IFF there wasn't perfect consensus.
-            RowRewardWeighted = self.GetWeight( AdjPrinComp * (Rep/mean(Rep)).T )
+            RowRewardWeighted = self.GetWeight(AdjPrinComp * (reputation / np.mean(reputation)).T)
 
-        #note: Rep/mean(Rep) is a correction ensuring Reputation is additive. Therefore, nothing can be gained by splitting/combining Reputation into single/multiple accounts.
+        #note: reputation/mean(reputation) is a correction ensuring Reputation is additive. Therefore, nothing can be gained by splitting/combining Reputation into single/multiple accounts.
               
-        #Freshly-Calculated Reward (Reputation) - Exponential Smoothing
-        #New Reward: RowRewardWeighted
-        #Old Reward: Rep
-        SmoothedR = Alpha*(RowRewardWeighted) + (1-Alpha)*Rep.T
+        # Freshly-Calculated Reward (Reputation) - Exponential Smoothing
+        # New Reward: RowRewardWeighted
+        # Old Reward: reputation
+        SmoothedR = Alpha*RowRewardWeighted + (1-Alpha)*reputation.T
 
-        Out = {
+        return {
             "FirstL": FirstLoading,
-            "OldRep": Rep.T,
+            "OldRep": reputation.T,
             "ThisRep": RowRewardWeighted,
             "SmoothRep":SmoothedR,
-        }  
-        return Out
+        }
 
     def GetDecisionOutcomes(self, Mtemp, ScaledIndex, Rep=-1, Verbose=False):
         """Determines the Outcomes of Decisions based on the provided reputation (weighted vote)."""
@@ -288,55 +281,54 @@ class Oracle(object):
             # Discriminate Based on Contract Type
             if not ScaledIndex[i]:
                 # Our Current best-guess for this Binary Decision (weighted average)
-                DecisionOutcomes_Raw.append(dot(Col, Row))
+                DecisionOutcomes_Raw.append(np.dot(Col, Row))
             else:
                 # Our Current best-guess for this Scaled Decision (weighted median)
-                wmed = self.WeightedMedian(Row[:,0], Col)
-                DecisionOutcomes_Raw.append()
+                wmed = weighted_median(Row[:,0], Col)
+                DecisionOutcomes_Raw.append(wmed)
 
-        return array(DecisionOutcomes_Raw).T
+        return np.array(DecisionOutcomes_Raw).T
 
-    def FillNa(self, Mna, ScaledIndex, Rep=-1, CatchP=.1, Verbose=False):
+    def FillNa(self, votes_na, ScaledIndex, reputation=-1, CatchP=.1, Verbose=False):
         """Uses exisiting data and reputations to fill missing observations.
         Essentially a weighted average using all availiable non-NA data.
         How much should slackers who arent voting suffer? I decided this would
         depend on the global percentage of slacking.
         """
-        if type(Rep) is int:
-            Rep = self.DemocracyCoin(Mat)
+        if type(reputation) is int:
+            reputation = self.DemocracyCoin(votes_na)
 
-        # In case no Missing values, Mnew and Mna will be the same.
-        Mnew = ma.copy(Mna)
+        # In case no Missing values, Mnew and votes_na will be the same.
+        votes_na_new = ma.copy(votes_na)
 
         # Of course, only do this process if there ARE missing values.
-        if sum(Mna.mask) > 0:
-            # Decision Outcome -
+        if votes_na.mask.any():
             # Our best guess for the Decision state (FALSE=0, Ambiguous=.5, TRUE=1)
             # so far (ie, using the present, non-missing, values).
-            DecisionOutcomes_Raw = self.GetDecisionOutcomes(Mna, ScaledIndex, Rep, Verbose).squeeze()
+            DecisionOutcomes_Raw = self.GetDecisionOutcomes(votes_na, ScaledIndex, reputation, Verbose).squeeze()
 
             # Fill in the predictions to the original M
-            NAmat = Mna.mask  # Defines the slice of the matrix which needs to be edited.
-            Mnew[NAmat] = 0  # Erase the NA's
+            NAmat = votes_na.mask  # Defines the slice of the matrix which needs to be edited.
+            votes_na_new[NAmat] = 0  # Erase the NA's
 
             # Slightly complicated:
-            NAsToFill = dot(NAmat, diag(DecisionOutcomes_Raw))
+            NAsToFill = np.dot(NAmat, np.diag(DecisionOutcomes_Raw))
             
             # This builds a matrix whose columns j:
             #   NAmat was false (the observation wasn't missing) - have a value of Zero
             #   NAmat was true (the observation was missing)     - have a value of the jth element of DecisionOutcomes.Raw (the 'current best guess')
-            Mnew = Mnew + NAsToFill
+            votes_na_new += NAsToFill
             # This replaces the NAs, which were zeros, with the predicted Decision outcome.
 
             # Appropriately force the predictions into their discrete
             # (0,.5,1) slot. (continuous variables can be gamed).
-            rows, cols = Mnew.shape
+            rows, cols = votes_na_new.shape
             for i in range(rows):
                 for j in range(cols):
                     if not ScaledIndex[j]:
-                        Mnew[i][j] = self.Catch(Mnew[i][j], CatchP)
+                        votes_na_new[i][j] = self.Catch(votes_na_new[i][j], CatchP)
 
-        return Mnew
+        return votes_na_new
 
     def consensus(self):
         """
@@ -387,14 +379,15 @@ class Oracle(object):
         # Consensus - "Who won?" Decision Outcome    
         # Simple matrix multiplication ... highest information density at RowBonus,
         # but need DecisionOutcomes.Raw to get to that
-        DecisionOutcomes_Raw = dot(PlayerInfo['SmoothRep'], Filled).squeeze()
+        DecisionOutcomes_Raw = np.dot(PlayerInfo['SmoothRep'], Filled).squeeze()
 
         # Discriminate Based on Contract Type
         ncols = Filled.shape[1]
         for i in range(ncols):
             # Our Current best-guess for this Scaled Decision (weighted median)
             if ScaledIndex[i]:
-                DecisionOutcomes_Raw[i] = self.WeightedMedian(Filled[:,i], PlayerInfo["SmoothRep"])
+                DecisionOutcomes_Raw[i] = weighted_median(Filled[:,i],
+                                                          PlayerInfo["SmoothRep"].flatten())
 
         # .5 is obviously undesireable, this function travels from 0 to 1
         # with a minimum at .5
@@ -404,7 +397,7 @@ class Oracle(object):
         ConReward = self.GetWeight(Certainty)
 
         # How well did beliefs converge?
-        Avg_Certainty = mean(Certainty)
+        Avg_Certainty = np.mean(Certainty)
 
         # The Outcome Itself
         # Discriminate Based on Contract Type
@@ -427,7 +420,7 @@ class Oracle(object):
 
         # Participation Within Decisions (Columns)
         # % of reputation that answered each Decision
-        ParticipationC = 1 - dot(PlayerInfo['SmoothRep'], NAmat)
+        ParticipationC = 1 - np.dot(PlayerInfo['SmoothRep'], NAmat)
 
         # Participation Within Agents (Rows)
         # Many options
@@ -435,9 +428,9 @@ class Oracle(object):
         ParticipationR = 1 - NAmat.sum(axis=1) / NAmat.shape[1]
 
         # General Participation
-        PercentNA = 1 - mean(ParticipationC)
+        PercentNA = 1 - np.mean(ParticipationC)
 
-        # (Possibly integrate two functions of participation?) Chicken and egg problem...
+        # Possibly integrate two functions of participation? Chicken and egg problem...
         if Verbose:
             print('*Participation Information*')
             print('Voter Turnout by question')
